

#include <windows.h>
#include "Pioneer1.h"

#define MAX_ROBOTS 10

enum MANEVEUR		{manAdvance,manBack,manRotateLeft,manRotateRight};
enum COLLISIONAREA  {bumNone, bumTop, bumTopLeft, bumBottomLeft, bumBottom, bumBottomRight, bumTopRight};


//---Global status variables

typedef struct  {
		enum COLLISIONAREA	LastCollisionArea;
		enum MANEVEUR Maneveur;
		int	ManTime;} CONTROL_STATUS;


CONTROL_STATUS Control[MAX_ROBOTS];




//---ROBOT.EXE functions --------------------------------
double (*pGetSensorValue)(int idRobot,int sensor,int var);
int (*pSendMsg)(UINT msg,WPARAM wParam0, WPARAM wParam1);

// Get a sensor value
double GetSensorValue(int idRobot,int sensor,int var)
{
	return (*pGetSensorValue)(idRobot,sensor,var);
}

// Send a order to robot
int SendMsg(UINT msg,WPARAM wParam0, WPARAM wParam1)
{
	return (*pSendMsg)(msg,wParam0,wParam1);
}


//---DLL entry --------------------------------


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	HMODULE hMod;

    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:

			
			hMod=GetModuleHandle("ROBOT.EXE");
			if(hMod)
			{
				pGetSensorValue=(double (*)(int,int,int)) GetProcAddress(hMod,"GetSensorValue");
				pSendMsg=(int(*)(UINT,WPARAM,WPARAM)) GetProcAddress(hMod,"SendMsg");
				if(!pGetSensorValue ||  !pSendMsg)
				{
					MessageBox(NULL,"Error in robot.exe","Error",MB_ICONERROR);
					return FALSE;
				}
			}
			break;


		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}




//---DLL export functions --------------------------------

DLL_API WORD OnProc(WPARAM wParam0,WPARAM wParam1)
{
	switch(wParam0)
	{
	case PRC_INIT:
		//Create system


		if(wParam1>MAX_ROBOTS)
		{
			MessageBox(NULL,"Only one robot is supported","Error",MB_ICONERROR);
			break;
		}
		memset(Control,0,sizeof(Control));

		break;


	}
	return 0;
}


DLL_API int CreateControlRobot(int idRobot,double xInicial,double yInicial,double OrientacioInicial,LPCSTR CodiControlador)
{

	return 0;
}




//----------------------------------------------------------
//   CONTROL ALGORITHM
//----------------------------------------------------------



// Return area of collision
enum COLLISIONAREA DetectCollision(int id)
{
	if(GetSensorValue(id,10,0) || GetSensorValue(id,11,0)) return bumTopRight;
	if(GetSensorValue(id,12,0)) return bumTop;
	if(GetSensorValue(id,13,0) || GetSensorValue(id,14,0)) return bumTopLeft;

	if(GetSensorValue(id,15,0) || GetSensorValue(id,16,0)) return bumBottomRight;
	if(GetSensorValue(id,17,0)) return bumBottom;
	if(GetSensorValue(id,18,0) || GetSensorValue(id,19,0)) return bumBottomLeft;

	return bumNone;
}

// Main function
DLL_API void RefreshControl(int idControl,int dt)
{
	double Left,Right;
	enum COLLISIONAREA col;


	if(Control[idControl].Maneveur!=manAdvance)
	{
		Control[idControl].ManTime-=dt;
		//new maneveur?
		if(Control[idControl].ManTime<0)
		{
			switch(Control[idControl].Maneveur)
			{
			case manBack:
				// Rotate for ManTime ms
				Control[idControl].Maneveur=Control[idControl].LastCollisionArea==bumTopLeft?
						manRotateRight : manRotateLeft;

				Control[idControl].ManTime=  (int) (200.0F*((float) rand() / RAND_MAX))+400;
				break;
			default:
				Control[idControl].Maneveur=manAdvance;
			}
		}
	}


	col=DetectCollision(idControl);

	if(col!=bumNone) Control[idControl].LastCollisionArea=col;

	//If collision area is top, then go back, rotate and advance
	if(Control[idControl].Maneveur!=manBack && (col==bumTop || col==bumTopLeft || col==bumTopRight))
	{
		Control[idControl].Maneveur=manBack;
		Control[idControl].ManTime=1400; //1.4s
	}

	//if go back and collision, then advance
	if(Control[idControl].Maneveur==manBack && (col==bumBottom || col==bumBottomLeft || col==bumBottomRight))
	{
			Control[idControl].Maneveur=manAdvance;
	}

		


	switch(Control[idControl].Maneveur)
	{
	case manBack:
		Left=-20;Right=-20;break;

	case manRotateLeft:
		Left=-10;Right=10;break;

	case manRotateRight:
		Left=10;Right=-10;break;

	case manAdvance:
	default:
		Left= 30;Right=30; break;
	}
	

	//Order to left and right robot motors
	SendMsg(MSG_MOTOR_LEFT,idControl,(WPARAM)Left);
	SendMsg(MSG_MOTOR_RIGHT,idControl,(WPARAM) Right);


} 


