// Sample1.cpp 
// A typical problem with autonomous robots occurs when the 
// robot needs to follow a path drawn on the ground.
// The controller's objective is to follow the path with a 
// maximum of speed and precision. A matrix of infrared ground sensors
// are used to detect the path, differentiating it from the floor 
// by colour contrast.
// Collision detectors, which stop the robot in the case 
// of possible collision, have been installed
// 
// Control keys:
//   Space: Stop 
//   Up:	Increase speed	   
//   Down:	Decrease speed	   


#include <windows.h>
#include "Sample1.h"


//---Global status variables
static double ant;
static double Left,Right;
static double Velocity;
static double pos;



//---ROBOT.EXE functions --------------------------------
double (*pGetSensorValue)(int idRobot,int sensor,int var);
int (*pSendMsg)(UINT msg,WPARAM wParam0, WPARAM wParam1);

// Get a sensor value
double GetSensorValue(int idRobot,int sensor,int var)
{
	return (*pGetSensorValue)(idRobot,sensor,var);
}

// Send a order to robot
int SendMsg(UINT msg,WPARAM wParam0, WPARAM wParam1)
{
	return (*pSendMsg)(msg,wParam0,wParam1);
}



//---DLL entry --------------------------------


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	HMODULE hMod;

    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:

			hMod=GetModuleHandle("ROBOT.EXE");
			if(hMod)
			{
				pGetSensorValue=(double (*)(int,int,int)) GetProcAddress(hMod,"GetSensorValue");
				pSendMsg=(int(*)(UINT,WPARAM,WPARAM)) GetProcAddress(hMod,"SendMsg");
				if(!pGetSensorValue ||  !pSendMsg)
				{
					MessageBox(NULL,"Error in robot.exe","Error",MB_ICONERROR);
					return FALSE;
				}
			}
			break;


		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}




//---DLL export functions --------------------------------

DLL_API WORD OnProc(WPARAM wParam0,WPARAM wParam1)
{
	switch(wParam0)
	{
	case PRC_INIT:
		//Create system
		ant=0;
		pos=-1;
		Velocity=20;

		if(wParam1!=1)
		{
			MessageBox(NULL,"Only one robot is supported","Error",MB_ICONERROR);
			break;
		}

		break;


	}
	return 0;
}


DLL_API int CreateControlRobot(int idRobot,double xInicial,double yInicial,double OrientacioInicial,LPCSTR CodiControlador)
{

	return 0;
}


DLL_API void OnChar(UINT nChar,BYTE down)
{
	switch(nChar)
	{

	case ' ':  //stop
		Velocity=0;
		break;

	case VK_DOWN: //decrease 
		Velocity--;
		break;

	case VK_UP:  //icrease
		Velocity++;
		break;
	}



}




DLL_API void RefreshControl(int idControl,int dt)
{
	double Left,Right;

	//if collision sensors are activated then stop
	if((GetSensorValue(idControl,5,0) || GetSensorValue(idControl,6,0)) && Velocity>0)
	{
		Velocity=0;
		SendMsg(MSG_MOTOR_LEFT,idControl,0);
		SendMsg(MSG_MOTOR_RIGHT,idControl,0);
		return;
		
	}



	//Sniffer sensors for navigation
	if(GetSensorValue(idControl,10,0))	pos=0.5;
	if(GetSensorValue(idControl,11,0)) pos=0.25;
	if(GetSensorValue(idControl,12,0)) pos=0;
	if(GetSensorValue(idControl,13,0)) pos=-0.25;
	if(GetSensorValue(idControl,14,0)) pos=-0.5;
	
	if(pos==-1) 
		pos=ant;
	else
		ant=pos;
	
	//Left and right velocity
	Left= Velocity*(1-pos*2.5);
	Right= Velocity*(1+pos*2.5);


	//Order to left and right robot motors
	SendMsg(MSG_MOTOR_LEFT,idControl,(WPARAM)Left);
	SendMsg(MSG_MOTOR_RIGHT,idControl,(WPARAM) Right);


} 


